@@ -16,10 +16,11 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2006 Dominic Lachowicz <cinamod@hotmail.com>
-// Copyright (C) 2007-2008, 2010 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007-2008, 2010, 2011 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2009 Jan Jockusch <jan@jockusch.de>
 // Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2010 Kenneth Berland <ken@hero.com>
+// Copyright (C) 2011 Tom Gleason <tom@buildadam.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -341,7 +342,7 @@ int main(int argc, char *argv[]) {
     if (textOut->isOk()) {
       fprintf(f, "<doc>\n");
       for (int page = firstPage; page <= lastPage; ++page) {
-        fprintf(f, "  <page width=\"%f\" height=\"%f\">\n",doc->getPageCropWidth(page), doc->getPageCropHeight(page));
+        fprintf(f, "  <page width=\"%f\" height=\"%f\">\n",doc->getPageMediaWidth(page), doc->getPageMediaHeight(page));
         doc->displayPage(textOut, page, resolution, resolution, 0, gTrue, gFalse, gFalse);
         TextWordList *wordlist = textOut->makeWordList();
         const int word_length = wordlist != NULL ? wordlist->getLength() : 0;
@@ -362,7 +363,6 @@ int main(int argc, char *argv[]) {
       fprintf(f, "</doc>\n");
     }
     fclose(f);
-    delete textOut;
   } else {
     textOut = new TextOutputDev(textFileName->getCString(),
 				physLayout, rawOrder, htmlMeta);
